unit mFMField;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  oFMField,
  oScriptTypes,
  Graphics,
  Controls,
  XMLIntf,
  Forms,
  oFMFile,
  Dialogs,
  StdCtrls;

type
  TfraFMField = class(TFrame)
    cbxFields: TComboBox;
    edtTag: TEdit;
    cbxFormat: TComboBox;
    edtFieldComment: TEdit;
    lblOrder: TLabel;
    cbxDataType: TComboBox;
    procedure cbxFieldsChange(Sender: TObject);
  private
    { Private declarations }
  public
    procedure Initialize(aXMLNode: IXMLNode);
    function getFMScriptLine: string;
    function getFMField: string;
    function getTag: string;
    function getFormat: TFMFieldFormat;
    function getDataType: string;
    function getComment: string;
    { Public declarations }
  end;

function NewFMFieldFrame(aOwner: TWinControl; aOnEnterEvent, aOnExitEvent: TNotifyEvent): TfraFMField;

implementation

uses
  StrUtils;

{$R *.dfm}

function NewFMFieldFrame(aOwner: TWinControl; aOnEnterEvent, aOnExitEvent: TNotifyEvent): TfraFMField;
begin
  Result := TfraFMField.Create(aOwner);
  Result.Parent := aOwner;
  Result.Name := 'fra_' + IntToStr(Random(10000000));
  Result.Top := aOwner.ComponentCount * Result.Height;
  Result.Align := alTop;
  Result.cbxFormat.ItemIndex := 0;
  Result.lblOrder.Font.Style := [fsBold];
  Result.OnEnter := aOnEnterEvent;
  Result.OnExit := aOnExitEvent;
end;

procedure TfraFMField.cbxFieldsChange(Sender: TObject);
begin
  edtTag.Text := AnsiReplaceStr(cbxFields.Text, ' ', '_');
  cbxFormat.ItemIndex := 0;
  if cbxFields.ItemIndex > -1 then
    with TFMField(cbxFields.Items.Objects[cbxFields.ItemIndex]) do
      edtFieldComment.Text := Format('Fileman field %s (#%s)', [Name, Number])
  else
    edtFieldComment.Text := '';
end;

function TfraFMField.getComment: string;
begin
  Result := edtFieldComment.Text;
end;

function TfraFMField.getDataType: string;
begin
  Result := cbxDataType.Text;
  (*
  if cbxFields.ItemIndex > -1 then
    Result := TFMField(cbxFields.Items.Objects[cbxFields.ItemIndex]).DataType
  else
    Result := 'FREE TEXT';
  *)
end;

function TfraFMField.getFMField: string;
begin
  if cbxFields.ItemIndex > -1 then
    Result := TFMField(cbxFields.Items.Objects[cbxFields.ItemIndex]).Number
  else
    Result := cbxFields.Text;
end;

function TfraFMField.getFMScriptLine: string;
begin
  if cbxFields.ItemIndex > -1 then
    with TFMField(cbxFields.Items.Objects[cbxFields.ItemIndex]) do
      Result := Format(
        '%s;%s;%s;%s;%s',
        [Number, edtTag.Text, Copy(cbxFormat.Text, 1, 1), cbxDataType.Text, edtFieldComment.Text])
  else
    Result := Format(
      '%s;%s;%s;%s;%s',
      [cbxFields.Text, edtTag.Text, Copy(cbxFormat.Text, 1, 1), cbxDataType.Text, edtFieldComment.Text]);
end;

function TfraFMField.getFormat: TFMFieldFormat;
begin
  Result := TFMFieldFormat(cbxFormat.ItemIndex);
end;

function TfraFMField.getTag: string;
begin
  Result := edtTag.Text;
end;

procedure TfraFMField.Initialize(aXMLNode: IXMLNode);
var
  i: integer;
begin
  cbxFields.ItemIndex := -1;
  for i := 0 to cbxFields.Items.Count - 1 do
    if TFMField(cbxFields.Items.Objects[i]).Number = aXMLNode.Attributes['FMFIELD'] then
      begin
        cbxFields.ItemIndex := i;
        Break;
      end;
  if cbxFields.ItemIndex < 0 then
    begin
      cbxFields.Text := aXMLNode.Attributes['FMFIELD'];
      //ShowMessage(aXMLNode.XML);
      //edtFunction.Text := aXMLNode.Attributes['FMFIELD'];
    end;
  edtTag.Text := aXMLNode.Attributes['TAG'];
  cbxFormat.ItemIndex := aXMLNode.Attributes['FORMAT'];
  edtFieldComment.Text := aXMLNode.Attributes['COMMENT'];
  cbxDataType.ItemIndex := cbxDataType.Items.IndexOf(aXMLNode.Attributes['DATATYPE']);
end;

end.

