unit oFMField;

interface

uses
  Classes,
  SysUtils,
  oScriptTypes,
  XMLDoc,
  XMLIntf;

type
  TFMField = class(TObject)
  private
    fNumber: string;
    fComment: string;
    fName: string;
    fFormat: TFMFieldFormat;
    fDataType: TFMFieldType;
    function getDescriptiveName: string;
    function getDefaultXMLTag: string;
  public
    property Name: string read fName write fName;
    property DefaultXMLTag: string read getDefaultXMLTag;
    property DescriptiveName: string read getDescriptiveName;
    property Number: string read fNumber write fNumber;
    property DataType: TFMFieldType read fDataType write fDataType;
    property OutputFormat: TFMFieldFormat read fFormat write fFormat;
    property Comment: string read fComment write fComment;
  end;

function NewFMField(aXMLNode: IXMLNode): TFMField;

implementation

uses StrUtils;

function NewFMField(aXMLNode: IXMLNode): TFMField;
var
  aType: TFMFieldType;
begin
  Result := TFMField.Create;
  with Result do
    begin
      fName := aXMLNode.Attributes['NAME'];
      fNumber := aXMLNode.Attributes['NUMBER'];

      for aType := Low(TFMFieldType) to High(TFMFieldType) do
        if Pos(aXMLNode.Attributes['TYPE'], FIELD_TYPE[aType]) > 0 then
          begin
            fDataType := aType;
            Exit;
          end;

      fDataType := ftFreeText;
    end;
end;

{ TFMField }

function TFMField.getDefaultXMLTag: string;
begin
  Result := AnsiReplaceText(fName, ' ', '_');
end;

function TFMField.getDescriptiveName: string;
begin
  Result := Format('%s (# %s)', [fName, fNumber]);
end;

end.

