unit uAddEdtDevice;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, uDevices, StdCtrls, ExtCtrls, Spin;

type
  TfDeviceEditor = class(TForm)
    lvObsTypes: TListView;
    btnOK: TButton;
    btnCancel: TButton;
    Bevel1: TBevel;
    btnAddNewObsType: TButton;
    Label2: TLabel;
    edtObsDesc: TEdit;
    Label3: TLabel;
    cbValueType: TComboBox;
    gbRange: TGroupBox;
    seHigh: TSpinEdit;
    seLow: TSpinEdit;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    edtObservationIdentifier: TEdit;
    edtObservationSubID: TEdit;
    Label7: TLabel;
    edtObservationValue: TEdit;
    Label8: TLabel;
    btnAbnormalFlags: TButton;
    btnAbnormalNature: TButton;
    Label1: TLabel;
    seObsFrequency: TSpinEdit;
    Label9: TLabel;
    gbValueOverride: TGroupBox;
    edtValueOverride: TEdit;
    Label10: TLabel;
    procedure btnOKClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure lvObsTypesClick(Sender: TObject);
    procedure btnAddNewObsTypeClick(Sender: TObject);
    procedure cbValueTypeSelect(Sender: TObject);
  private
    { Private declarations }
    FDevice: TDevice;
    procedure EnableUI;
    procedure DisableUI;
    procedure LoadListView;
  public
    { Public declarations }
    procedure ApplyToObsType;
  published
    property Device: TDevice read FDevice write FDevice;
  end;

var
  fDeviceEditor: TfDeviceEditor;

implementation

{$R *.dfm}

procedure TfDeviceEditor.btnOKClick(Sender: TObject);
begin
  // Assign all of the device and observation properties.
  ApplyToObsType;
  // Then...
  ModalResult := mrOK;
end;

procedure TfDeviceEditor.btnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TfDeviceEditor.FormShow(Sender: TObject);
begin
  if not assigned(FDevice) then
    raise Exception.Create('Device editor unable to display nil device');
  Caption := 'Add/Edit ' + FDevice.Description + ' device...';
  LoadListView;
end;

procedure TfDeviceEditor.lvObsTypesClick(Sender: TObject);
begin
  if Assigned(lvObsTypes.Selected) then
    EnableUI
  else
    DisableUI;
end;

procedure TfDeviceEditor.DisableUI;
begin
  edtObsDesc.Enabled := false;
  cbValueType.Enabled := false;
  seHigh.Enabled := false;
  seLow.Enabled := false;
  edtObservationIdentifier.Enabled := false;
  edtObservationSubID.Enabled := false;
  edtObservationValue.Enabled := false;
  seObsFrequency.Enabled := false;
end;

procedure TfDeviceEditor.EnableUI;
var
  Obs: TObservation;
begin
  if not Assigned(lvObsTypes.Selected) then
    exit;

  Obs := TObservation(lvObsTypes.Selected.Data);


  edtObsDesc.Text := Obs.Description;
  cbValueType.ItemIndex := ord(Obs.ValueType);
  seHigh.Value := Round(Obs.RangeHi);
  seLow.Value := Round(Obs.RangeLo);
  edtObservationIdentifier.Text := Obs.ObservationIdentifier;
  edtObservationSubID.Text := Obs.ObservationSubID;
  edtObservationValue.Text := Obs.ObservationValue;
  edtValueOverride.Text := Obs.DirectValue;
  seObsFrequency.Value := Round(Obs.Frequency/1000);

  edtObsDesc.Enabled := true;
  cbValueType.Enabled := true;
  edtObservationIdentifier.Enabled := true;
  edtObservationValue.Enabled := true;
  edtObservationSubID.Enabled := true;
  seObsFrequency.Enabled := true;
  edtValueOverride.Enabled := true;
  cbValueTypeSelect(nil);
end;

procedure TfDeviceEditor.btnAddNewObsTypeClick(Sender: TObject);
var
  NewName: string;
  Obs: TObservation;
begin
  if InputQuery('New Observation Type...', 'Please enter a name for a new observation type', NewName) then
  begin
    Obs := FDevice.Observations.Add;
    Obs.Description := NewName;
    LoadListView;
  end;
end;

procedure TfDeviceEditor.ApplyToObsType;
var
  Obs: TObservation;
begin
  if Assigned(lvObsTypes.Selected) and Assigned(lvObsTypes.Selected.Data) then
  begin
    if TObject(lvObsTypes.Selected.Data) is TObservation then
    begin
      //Start copying all of the fields into the Observation.
      Obs := TObservation(lvObsTypes.Selected.Data);
      Obs.Description := edtObsDesc.Text;
      Obs.ValueType := TValueType(cbValueType.ItemIndex);
      if gbRange.Enabled then
      begin
        Obs.RangeHi := seHigh.Value;
        Obs.RangeLo := seLow.Value;
      end
      else
      begin
        Obs.RangeHi := 0;
        Obs.RangeLo := 0;
      end;
      Obs.ObservationIdentifier := edtObservationIdentifier.Text;
      Obs.ObservationSubID := edtObservationSubID.Text;
      Obs.ObservationValue := edtObservationValue.Text;
      Obs.Frequency := seObsFrequency.Value * 1000;
      Obs.DirectValue := edtValueOverride.Text;
    end;
  end;
end;

procedure TfDeviceEditor.cbValueTypeSelect(Sender: TObject);
begin
  gbRange.Enabled := (cbValueType.ItemIndex = 10) or (cbValueType.ItemIndex = 11);
  edtObservationValue.Enabled := not gbRange.Enabled;
end;

procedure TfDeviceEditor.LoadListView;
var
  I : Integer;
  Item: TListItem;
begin
  lvObsTypes.Items.BeginUpdate;
  try
    lvObsTypes.Clear;
    for I := 0 to FDevice.Observations.Count - 1 do
    begin
      Item := lvObsTypes.Items.Add;
      Item.Caption := FDevice.Observations[I].Description;
      Item.Data := FDevice.Observations[I];
    end;
  finally
    lvObsTypes.Items.EndUpdate;
  end;
end;

end.

