unit uPatients;

interface

uses
  Classes, SysUtils, IniFiles;

type
  TAddress = class(TObject)
  private
    FAddress1: string;
    FAddress2: string;
    FCity: string;
    FState: string;
    FZip: string;
  protected
  public
  published
    property Address1: string read FAddress1 write FAddress1;
    property Address2: string read FAddress2 write FAddress2;
    property City: string read FCity write FCity;
    property State: string read FState write FState;
    property Zip: string read FZip write FZip;
  end;

  TPatientList = class;
  TPatient = class(TCollectionItem)
  private
    FInvalidData: string;
    FID: string;
    FFirstName: string;
    FMiddleName: string;
    FLastName: string;
    FSSAN: string;
    FDFN: string;
    FMothersMaidenName: string;
    FDateOfBirth: TDateTime;
    FSex: string;
    FAlias: string;
    FRace: string;
    FAddress: TAddress;
  protected
    function GetID: string;
    function GetSegment: string;
    function GetFullName: string;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
  published
    property InvalidData: string read FInvalidData write FInvalidData;
    property ID: string read GetID;
    property FirstName: string read FFirstName write FFirstName;
    property MiddleName: string read FMiddleName write FMiddleName;
    property LastName: string read FLastName write FLastName;
    property FullName: string read GetFullName;
    property SSAN: string read FSSAN write FSSAN;
    property DFN: string read FDFN write FDFN;
    property MothersMaidenName: string read FMothersMaidenName write FMothersMaidenName;
    property DateOfBirth: TDateTime read FDateOfBirth write FDateOfBirth;
    property Sex: string read FSex write FSex;
    property Alias: string read FAlias write FAlias;
    property Race: string read FRace write FRace;
    property Address: TAddress read FAddress write FAddress;
    property Segment: string read GetSegment;
  end;

  TPatientList = class(TCollection)
  private
    FHashList: THashedStringList;
  protected
    function GetItem(Index: Integer): TPatient;
    procedure SetItem(Index: Integer; Value: TPatient);
  public
    constructor Create(ItemClass: TCollectionItemClass);
    destructor Destroy; override;
    function Add: TPatient;
    function Find(AID: string): TPatient;
    function Insert(Index: Integer): TPatient;
    procedure Load(AFilename: string);
    procedure Save(AFilename: string);
    property Items[Index: Integer]: TPatient read GetItem write SetItem; default;
  published
  end;

implementation

uses
  XMLDoc, XMLIntf, XMLDOM, Variants, ComObj, ActiveX, uConstants;

{ TPatientList }

function TPatientList.Add: TPatient;
begin
  result := TPatient(Inherited Add);
end;

constructor TPatientList.Create(ItemClass: TCollectionItemClass);
begin
  inherited Create(ItemClass);
  FHashList := THashedStringList.Create;
end;

destructor TPatientList.Destroy;
begin
  FreeAndNil(FHashList);
  inherited;
end;

function TPatientList.Find(AID: string): TPatient;
begin
  result := TPatient(FHashList.Objects[FHashList.IndexOfName(AID)]);
end;

function TPatientList.GetItem(Index: Integer): TPatient;
begin
  result := TPatient(inherited GetItem(Index));
end;

function TPatientList.Insert(Index: Integer): TPatient;
begin
  result := TPatient(inherited Insert(Index));
end;

procedure TPatientList.Load(AFilename: string);
var
  XMLDoc: IXMLDocument;
  I: Integer;
  PatientNode: IXMLNode;
  ChildNode: IXMLNode;
  DataNode: IXMLNode;
  AddressNode: IXMLNode;
  Patient: TPatient;
begin
  Clear;
  XMLDoc := TXMLDocument.Create(nil);
  try
    DefaultDOMVendor := 'MSXML';

    XMLDoc.LoadFromFile(AFilename);
    XMLDoc.Options := XMLDoc.Options + [doNodeAutoIndent];
    XMLDoc.Active := true;
    XMLDoc.Encoding := 'UTF-8';

    PatientNode := XMLDoc.DocumentElement;

    for I := 0 to PatientNode.ChildNodes.Count - 1 do
    begin
      Patient := Add;
      ChildNode := PatientNode.ChildNodes[i];

      DataNode := ChildNode.ChildNodes.FindNode(ID);
      if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
      begin
        Patient.FID := DataNode.NodeValue;
        FHashList.AddObject(Patient.FID + '=', Patient);
      end;
        
      DataNode := ChildNode.ChildNodes.FindNode(INVALIDDATA);
      if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
        Patient.FInvalidData := DataNode.NodeValue;

      DataNode := ChildNode.ChildNodes.FindNode(FIRSTNAME);
      if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
        Patient.FFirstName := DataNode.NodeValue;

      DataNode := ChildNode.ChildNodes.FindNode(MIDDLENAME);
      if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
        Patient.FMiddleName := DataNode.NodeValue;

      DataNode := ChildNode.ChildNodes.FindNode(LASTNAME);
      if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
        Patient.FLastName := DataNode.NodeValue;

      DataNode := ChildNode.ChildNodes.FindNode(DFN);
      if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
        Patient.FDFN := DataNode.NodeValue;

      DataNode := ChildNode.ChildNodes.FindNode(SSAN);
      if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
        Patient.FSSAN := DataNode.NodeValue;

      DataNode := ChildNode.ChildNodes.FindNode(MOTHERSMAIDENNAME);
      if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
        Patient.FMothersMaidenName := DataNode.NodeValue;

      DataNode := ChildNode.ChildNodes.FindNode(DOB);
      if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
        Patient.FDateOfBirth := StrToDateTime(DataNode.NodeValue);

      DataNode := ChildNode.ChildNodes.FindNode(SEX);
      if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
        Patient.FSex := DataNode.NodeValue;

      DataNode := ChildNode.ChildNodes.FindNode(ALIAS);
      if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
        Patient.FAlias := DataNode.NodeValue;

      DataNode := ChildNode.ChildNodes.FindNode(RACE);
      if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
        Patient.FRace := DataNode.NodeValue;

      AddressNode := ChildNode.ChildNodes.FindNode(ADDRESS);
      if assigned(AddressNode) then
      begin
        DataNode := AddressNode.ChildNodes.FindNode(STREET1);
        if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
          Patient.FAddress.FAddress1 := DataNode.NodeValue;

        DataNode := AddressNode.ChildNodes.FindNode(STREET2);
        if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
          Patient.FAddress.FAddress2 := DataNode.NodeValue;

        DataNode := AddressNode.ChildNodes.FindNode(CITY);
        if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
          Patient.FAddress.FCity := DataNode.NodeValue;

        DataNode := AddressNode.ChildNodes.FindNode(STATE);
        if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
          Patient.FAddress.FState := DataNode.NodeValue;

        DataNode := AddressNode.ChildNodes.FindNode(ZIP);
        if assigned(DataNode) and (DataNode.NodeValue <> NULL) then
          Patient.FAddress.FZip := DataNode.NodeValue;
      end;
    end;
  finally
    XMLDoc := nil;
  end;
end;

procedure TPatientList.Save(AFilename: string);
var
  XMLDoc: IXMLDocument;
  RootNode: IXMLNode;
  I: Integer;
  PatientNode: IXMLNode;
  ChildNode: IXMLNode;
  AddressNode: IXMLNode;
begin
  XMLDoc := TXMLDocument.Create(nil);
  try
    DefaultDOMVendor := 'MSXML';

    XMLDoc.Options := XMLDoc.Options + [doNodeAutoIndent];
    XMLDoc.Active := true;
    XMLDoc.Encoding := 'UTF-8';

    RootNode := XMLDoc.CreateElement(PATIENTS, '');
    XMLDoc.DocumentElement := RootNode;

    for I := 0 to Count - 1 do
    begin
      PatientNode := RootNode.AddChild(PATIENT);

      ChildNode := PatientNode.AddChild(ID);
      ChildNode.NodeValue := Items[I].GetID;
      ChildNode := PatientNode.AddChild(INVALIDDATA);
      ChildNode.NodeValue := Items[I].FInvalidData;
      ChildNode := PatientNode.AddChild(FIRSTNAME);
      ChildNode.NodeValue := Items[I].FFirstName;
      ChildNode := PatientNode.AddChild(MIDDLENAME);
      ChildNode.NodeValue := Items[I].FMiddleName;
      ChildNode := PatientNode.AddChild(LASTNAME);
      ChildNode.NodeValue := Items[I].FLastName;
      ChildNode := PatientNode.AddChild(DFN);
      ChildNode.NodeValue := Items[I].FDFN;
      ChildNode := PatientNode.AddChild(MOTHERSMAIDENNAME);
      ChildNode.NodeValue := Items[I].FMothersMaidenName;
      ChildNode := PatientNode.AddChild(SSAN);
      ChildNode.NodeValue := Items[I].FSSAN;
      ChildNode := PatientNode.AddChild(DOB);
      ChildNode.NodeValue := Items[I].FDateOfBirth;
      ChildNode := PatientNode.AddChild(SEX);
      ChildNode.NodeValue := Items[I].FSex;
      ChildNode := PatientNode.AddChild(ALIAS);
      ChildNode.NodeValue := Items[I].FAlias;
      ChildNode := PatientNode.AddChild(RACE);
      ChildNode.NodeValue := Items[I].FRace;
      AddressNode := PatientNode.AddChild(ADDRESS);
      ChildNode := AddressNode.AddChild(STREET1);
      ChildNode.NodeValue := Items[I].FAddress.FAddress1;
      ChildNode := AddressNode.AddChild(STREET2);
      ChildNode.NodeValue := Items[I].FAddress.FAddress2;
      ChildNode := AddressNode.AddChild(CITY);
      ChildNode.NodeValue := Items[I].FAddress.FCity;
      ChildNode := AddressNode.AddChild(STATE);
      ChildNode.NodeValue := Items[I].FAddress.FState;
      ChildNode := AddressNode.AddChild(ZIP);
      ChildNode.NodeValue := Items[I].FAddress.FZip;
    end;
    XMLDoc.SaveToFile(AFilename);
  finally
    XMLDoc := nil;
  end;
end;

procedure TPatientList.SetItem(Index: Integer; Value: TPatient);
begin
  inherited SetItem(Index,Value);
end;

{ TPatient }

constructor TPatient.Create(Collection: TCollection);
begin
  inherited;
  FAddress := TAddress.Create;
end;

destructor TPatient.Destroy;
begin
  FreeAndNil(FAddress);
  inherited;
end;

function TPatient.GetFullName: string;
begin
  result := FLastName + ', ' + FFirstName + ' ' + FMiddleName;
end;

function TPatient.GetID: string;
var
  GUID: TGUID;
begin
  if Trim(FID) = '' then
  begin
    CoCreateGUID(GUID);
    FID := GUIDToString(GUID);
  end;

  result := FID;
end;

function TPatient.GetSegment: string;
begin
  if Trim(FInvalidData) <> '' then
    result := FInvalidData
  else
    result :=
      'PID|' +
      '1||' +
      FSSAN +'|' +
      '|' +
      FLastName + '^' +
      FFirstName + '^' +
      FMiddleName + '|' +
      FMothersMaidenName + '|' +
      FormatDateTime('yyyymmdd', FDateOfBirth) + '|' +
      FSex + '||' +
      FRace + '|||||||||' +
      FSSAN + '||';
end;

initialization
  CoInitialize(nil);

finalization
  CoUninitialize;

end.
